###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################
from obt import dep, path, pathtools, wget, env
from obt.deco import Deco 
deco = Deco()
###############################################################################
class astrocom(dep.StdProvider):
  NAME = "astrocom"
  def __init__(self):
    super().__init__(astrocom.NAME)
    self.declareDep("cmake")
    self._builder = self.createBuilder(dep.CustomBuilder)
    self._builder._cleanOnClean = False
    self.build_dir = path.builds()/astrocom.NAME
    self.share_dir = path.share()/"astro"
    env = {
    	"PREFIX": path.stage()
    }

    def grab_starmaps():
      print(deco.yellow("fetching starmap data [2GPixel]"))
      smap_2G = wget.wget(urls=["https://svs.gsfc.nasa.gov/vis/a000000/a004800/a004851/starmap_2020_64k.exr"],
                           output_name=self.share_dir/"starmap_2020_64k.exr",
                           md5val="9f49029b21949fa57d6915868295d920")
      print(deco.yellow("fetching starmap data [512Mpixel]"))
      smap_512M = wget.wget(urls=["https://svs.gsfc.nasa.gov/vis/a000000/a004800/a004851/starmap_2020_32k.exr"],
                           output_name=self.share_dir/"starmap_2020_32k.exr",
                           md5val="726edf8a69b805379241cc5dc54653cd")
      print(deco.yellow("fetching starmap data [128Mpixel]"))
      smap_128M = wget.wget(urls=["https://svs.gsfc.nasa.gov/vis/a000000/a004800/a004851/starmap_2020_16k.exr"],
                           output_name=self.share_dir/"starmap_2020_16k.exr",
                           md5val="761e43a304db583f7d3955b57680fa8e")
      print(deco.yellow("fetching starmap data [8Mpixel]"))
      smap_8M = wget.wget(urls=["https://svs.gsfc.nasa.gov/vis/a000000/a004800/a004851/starmap_2020_4k.exr"],
                           output_name=self.share_dir/"starmap_2020_4k.exr",
                           md5val="eac33e4c28e42cfdb8dc26b3b9a7eefb")


      print(deco.yellow("fetching halpha healpix map"))
      halpha_map_1K = wget.wget(urls=["https://faun.rc.fas.harvard.edu/dfink/skymaps/halpha/data/v1_1/healpix/Halpha_fwhm06_1024.fits"],
                                output_name=self.share_dir/"Halpha_fwhm06_1024.fits",
                                md5val="7aca800d81a6cf1609dea08850b48c32")
      
      print(deco.yellow("fetching halpha healpix mask"))
      halpha_mask_1K = wget.wget(urls=["https://faun.rc.fas.harvard.edu/dfink/skymaps/halpha/data/v1_1/healpix/Halpha_mask_fwhm06_1024.fits"],
                                 output_name=self.share_dir/"Halpha_mask_fwhm06_1024.fits",
                                 md5val="95a675d45b06cfd03500a7e02e1974fc")
      
      print(deco.yellow("fetching halpha healpix error"))
      halpha_error_1K = wget.wget(urls=["https://faun.rc.fas.harvard.edu/dfink/skymaps/halpha/data/v1_1/healpix/Halpha_error_fwhm06_1024.fits"],
                                  output_name=self.share_dir/"Halpha_error_fwhm06_1024.fits",
                                  md5val="612d560c9b3ebe4ea2d8b18f6ef26c1f")
      
      print(deco.yellow("fetching halpha fullsky map"))
      halpha_map_32M = wget.wget(urls=["https://faun.rc.fas.harvard.edu/dfink/skymaps/halpha/data/v1_1/maps/Halpha_map.fits"],
                                 output_name=self.share_dir/"Halpha_map.fits",
                                 md5val="c174b17ca83922910ea21a9ead84ef8b")
      
      print(deco.yellow("fetching halpha fullsky mask"))
      halpha_mask_32M = wget.wget(urls=["https://faun.rc.fas.harvard.edu/dfink/skymaps/halpha/data/v1_1/maps/Halpha_mask.fits"],
                                  output_name=self.share_dir/"Halpha_mask.fits",
                                  md5val="7a832c8549c3b8f6a6fb36541b255278")
      
      print(deco.yellow("fetching halpha fullsky error"))
      halpha_error_32M = wget.wget(urls=["https://faun.rc.fas.harvard.edu/dfink/skymaps/halpha/data/v1_1/maps/Halpha_error.fits"],
                                   output_name=self.share_dir/"Halpha_error.fits",
                                   md5val="679aa35be7f2c7a531859f0f29010f7e")
                      
    cmdlist_incr = []
    cmdlist_incr += [ pathtools.mkdir(self.share_dir)]
    cmdlist_incr += [ dep.CustomStep("grab field", grab_starmaps) ]

    self._builder._incrbuildcommands = cmdlist_incr
    self._builder.setEnvVars(env)

  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=astrocom.NAME,
                             repospec="tweakoz/AstroComPYute",
                             revision="toz-2024-ub24",
                             recursive=True)

  ########################################################################
  def env_init(self):
    env.set("OBT_ASTROREF_DATA",self.share_dir)
  ########################################################################
  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"README.md").exists()
  



#!/usr/bin/env python3
###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################


import os, sys, pathlib, argparse, multiprocessing

as_main = (__name__ == '__main__')

###########################################

Path = pathlib.Path
curwd = Path(os.getcwd())
file_path = os.path.realpath(__file__)
file_dir = os.path.dirname(file_path)
par2_dir = os.path.dirname(file_dir)
root_dir = Path(par2_dir)

###########################################

parser = argparse.ArgumentParser(description='obt.build environment launcher')
parser.add_argument('--stagedir', metavar="stagedir", help='launch from pre-existing folder' )
parser.add_argument('--project', action='append', metavar="PROJECTLIST", help='append project directory' )
parser.add_argument('--prompt', metavar="prompt", help='prompt suffix' )
parser.add_argument("--numcores", metavar="numcores", help="numcores for environment")
parser.add_argument("--quiet", action="store_true", help="no output")
parser.add_argument('--novars', action="store_true", help='do not set env vars' )
parser.add_argument('--obttrace',action="store_true",help='enable OBT buildtrace logging')

parser.add_argument('--subspace', metavar="subspace", help='subspace to launch' )
parser.add_argument('--chdir', metavar="chdir", help='working directory of command' )
parser.add_argument("--command", metavar="command", help="execute in environ")
parser.add_argument('--stack', metavar="stackdir", help='stack env' )

args = vars(parser.parse_args())

if len(sys.argv)==1:
    print(parser.format_usage())
    sys.exit(1)

###########################################

from _obt_config import configFromCommandLine, initializeDependencyEnvironments
obt_config = configFromCommandLine(args)

###########################################

if args["obttrace"]==True:
  import obt._globals as _glob
  _glob.enableBuildTracing()

###########################################

import obt._envutils 
envsetup = obt._envutils.EnvSetup(obt_config)

###########################################

import obt._globals as _glob
import obt.command
import obt.deco
deco = obt.deco.Deco()

stage_dir = obt_config.stage_dir

###########################################

initializeDependencyEnvironments(envsetup)

###########################################
if args["stagedir"]!=None:
###########################################
    envsetup.lazyMakeDirs()
    envsetup.genBashRc(obt_config,stage_dir/".bashrc")
    stage_dir_sh = stage_dir/"obt-launch-env"
    envsetup.log(stage_dir_sh)
    assert(stage_dir_sh.exists())
    #############
    shell = "bash"
    bashrc = stage_dir/".bashrc"
    #############
    if args["subspace"]!=None:
        if args["chdir"]!=None:
            os.chdir(args["chdir"])
        #rval = os.system(args["command"]) # call shell with new vars (just "exit" to exit)
        subspacemodulename = args["subspace"]
        obt._globals.setOption("subspacemodulename",subspacemodulename)

        subspacemodule = obt.subspace.requires(subspacemodulename)

        if args["command"]!=None:
          if args["chdir"]!=None:
            os.chdir(args["chdir"])
          rval = subspacemodule.launch(obt.command.procargs(args["command"]))
        else:
          rval = subspacemodule.shell()
  
        sys.exit(rval>>8)
    #############
    elif args["command"]!=None:
        if args["chdir"]!=None:
            os.chdir(args["chdir"])
        rval = os.system(args["command"]) # call shell with new vars (just "exit" to exit)
        sys.exit(rval>>8)
    #############
    else:
        obt.command.Command([shell,"--init-file",bashrc],environment={}).exec()
###########################################
elif args["stack"]!=None:
###########################################
    envsetup.lazyMakeDirs()
    envsetup.genBashRc(obt_config,stage_dir/".bashrc-stack")
    #############
    if args["chdir"]!=None:
      os.chdir(args["chdir"])
    #############
    shell = "bash"
    bashrc = stage_dir/".bashrc-stack"
    envsetup.log(deco.inf("System is <"+os.name+">"))
    print("Stacking env<%s>" % deco.val(stage_dir))
    envsetup.log("obt.build eviron initialized OBT_ROOT<%s>"%deco.path(root_dir))
    if args["command"]!=None:
        obt.command.Command([shell,"--init-file",bashrc,"-c",args["command"]],environment={}).exec()
    else:
        obt.command.Command([shell,"--init-file",bashrc],environment={}).exec()
    pass
###########################################
else:
###########################################
    assert(False)



class sdkinfo:
  def __init__(self):
    self.identifier = "cuda"
    self.architecture = "cuda"
    #self.c_compiler = "clang"
    #self.cxx_compiler = "clang++"
    self.supports_host = ["x86_64-linux","aarch64-linux"]




  
###############################################################################

class project:

  #####################

  def __init__(self,named):
    self.name = named
    self.depends = []
    self.sources = []

  #####################

  def add_src(self,src):
    self.sources.append(src)

  #####################

  def dependsOn(self,project):
    self.depends.append(project)

  #####################
  
  def __repr__(self):
    return f"project({self.name})"
  
###############################################################################

class executable(project):
  def __init__(self,named):
    super().__init__(named)

  def emit(self):
    rval = []
    out_str = "add_executable(%s " % (self.name)
    for src in self.sources:
      out_str += src + " "
    out_str += ")"
    rval += [out_str]
    return rval

class sharedLibrary(project):
  def __init__(self,named):
    super().__init__(named)
  def emit(self):
    rval = []
    out_str = "add_library(%s SHARED " % (self.name)
    for src in self.sources:
      out_str += src + " "
    out_str += ")"
    rval += [out_str]
    return rval

###############################################################################

class emitter:
  def __init__(self, platform):
    self.platform = platform
    self.is_ios = "ios" in platform
    self.is_android = "android" in platform
  
###############################################################################

class libref:
  def __init__(self, named, REQUIRED=True, CONFIG=False):
    self.name = named
    self.required = REQUIRED
    self.config = CONFIG
    
  def emit(self):
    if self.required:
      return ['find_library(%s %s REQUIRED)' % (self.name,self.name)]
    else:
      return ['find_library(%s %s)' % (self.name,self.name)]
  
class pkgref:
  def __init__(self, named, REQUIRED=True, CONFIG=False, COMPONENTS=[]):
    self.name = named
    self.required = REQUIRED
    self.config = CONFIG
    self.components = COMPONENTS

  def emit(self):

    out_str = "find_package( " + self.name + " "
    if self.config:
      out_str += "CONFIG "
    if self.required:
      out_str += "REQUIRED "
    if len(self.components) > 0:
      out_str += "COMPONENTS "
      out_str += " ".join(self.components)
      
    out_str += ")"

    return [out_str]
###############################################################################
  
class workspace:

  #####################

  def __init__(self, named):
    self.name = named
    self.projects = dict()
    self.librefs = dict()
    self.pkgrefs = dict()
    self.vars = dict()
    pass

  #####################

  def findLibrary(self, named, REQUIRED=True, CONFIG=False):
    lref = libref(named,REQUIRED,CONFIG)
    self.librefs[named] = lref

  #####################

  def findPackage(self, named, #
                        REQUIRED=True, #
                        CONFIG=False, #
                        COMPONENTS=[]): #

    lref = pkgref(named, #
                  REQUIRED=REQUIRED, #
                  CONFIG=CONFIG, #
                  COMPONENTS=COMPONENTS)

    self.pkgrefs[named] = lref

  #####################

  def setVar(self, named, value):
    self.vars[named] = value

  #####################

  def createExecutable(self,named):
    p = executable(named)
    self.projects[named] = p
    return p
  
  def createSharedLibrary(self,named):
    p = sharedLibrary(named)
    self.projects[named] = p
    return p

  #####################

  def toposort(self):
    # topological sort of projects based on dependencies

    # 1. recursive function to visit a node, add to dictionary with depth as key
    # 2. sort the dictionary by depth
    # 3. return the reverse sorted list of projects (highest depths first)

    def visit(node,depth,visited,sorted):
      if node in visited:
        return
      visited.add(node)
      for dep in node.depends:
        visit(dep,depth+1,visited,sorted)
      if depth not in sorted:
        sorted[depth] = []
      sorted[depth].append(node)
      
    visited = set()
    sorted = dict()
    for p in self.projects.values():
      visit(p,0,visited,sorted)
      
    the_sorted = []
    for d in sorted.keys():
      the_sorted.extend(sorted[d])
      
    return the_sorted

  #####################

  def emit(self, emit_ctx):
    sorted = self.toposort()
    print(sorted)
    
    self.top_cmlist = []
    self.top_cmlist += ["cmake_minimum_required(VERSION 3.28.3)"]
    self.top_cmlist += ["set(CMAKE_CXX_STANDARD 20)"]
    self.top_cmlist += ["set(CMAKE_CXX_STANDARD_REQUIRED ON)"]
    self.top_cmlist += ["set(CMAKE_CXX_EXTENSIONS OFF)"]
    self.top_cmlist += ["project("+self.name+" LANGUAGES CXX OBJCXX)"]
    
    if emit_ctx.is_ios:
      #
      self.top_cmlist += ['set(CMAKE_OSX_SYSROOT %s CACHE PATH "The iOS SDK sysroot")']
      self.top_cmlist += ['include(${CMAKE_CURRENT_SOURCE_DIR}/../../ios.toolchain.cmake)']
      self.top_cmlist += ['include(${CMAKE_CURRENT_SOURCE_DIR}/conan_toolchain.cmake)']

    for k in self.librefs:
      lref = self.librefs[k]
      self.top_cmlist += lref.emit()
        
    for k in self.pkgrefs:
      pref = self.pkgrefs[k]
      self.top_cmlist += pref.emit()

    for k in self.vars:
      self.top_cmlist += ["set(%s %s)" % (k,self.vars[k])]
    
    for p in sorted:
      self.top_cmlist += p.emit()
      
    out_str = "\n".join(self.top_cmlist)
    print(out_str)
  


###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

VERSION = "v4.3.5"

import os, tarfile
from obt import dep, host, path

###############################################################################

class zmq(dep.StdProvider):
  name = "zmq"
  def __init__(self): ############################################
    super().__init__(zmq.name)
    self.declareDep("cmake")
    self._builder = self.createBuilder(dep.CMakeBuilder)
    if host.IsDarwin:
      self._builder.setCmVar("WITH_TLS","OFF")
      libpath = path.libs()/"libzmq.5.dylib"
      def postInstall():
        from obt import macos
        from obt.deco import Deco
        deco = Deco()
        print(deco.inf("Macos fixup dll installname for %s"% str(libpath)))
        macos.macho_change_id(libpath,"@rpath/libzmq.5.dylib")
      self._builder._onPostInstall = postInstall
  ########################################################################
  def __str__(self): 
    return "zeromq (github-%s)" % VERSION
  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=zmq.name,
                             repospec="zeromq/libzmq",
                             revision=VERSION,
                             recursive=False)
  ########################################################################
  def linkenv(self):
    LIBS = ["zmq"]
    return {
        "LIBS": LIBS,
        "LFLAGS": ["-l%s"%item for item in LIBS]
    }
  ########################################################################
  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"CMakeLists.txt").exists()
  def areRequiredBinaryFilesPresent(self):
    return (path.includes()/"zmq.h").exists()



###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

from obt.command import Command
from obt import path, dep, buildtrace
import os 

class context:

  ###############################################

  def __init__(self,
               root="",
               env=dict(),
               osenv=dict(),
               trace=False,
               sourcedir=None,
               builddir=None,
               working_dir=None,
               xcode=False,
               install_prefix=None,
               modules_paths=[]):

    self.root = root
    self.env = env
    self.osenv = osenv
    self._verbose = False
    self._trace = trace
    self._builddir = builddir 
    self._working_dir = working_dir
    self._sourcedir = sourcedir
    self._xcode = xcode
    self._install_prefix = install_prefix
    self._modules_paths = modules_paths

    if builddir!=None:
      self._sourcedir = builddir/".."
    if sourcedir!=None:
      self._sourcedir = sourcedir

  ###############################################

  @property 
  def install_prefix(self):
    return path.prefix() if (self._install_prefix==None) else self._install_prefix

  ###############################################

  def verbose(self,enable):
    self._verbose = enable

  ###############################################

  def exec(self):

    cmdlist = ["cmake"]

    if self._builddir!=None:
      cmdlist += [ "-B", self._builddir ]

    cmdlist += [ "-S", self._sourcedir ]

    if self._verbose:
      cmdlist += ["--verbose"]

    cmdlist += ["-DCMAKE_INSTALL_PREFIX=%s"%self.install_prefix]
    
    mpath = [
      self.install_prefix/"lib"/"cmake"
    ] + self._modules_paths
    
    mpathstr = "-DCMAKE_MODULE_PATH=%s"%":".join([str(p) for p in mpath])
    
    cmdlist += [mpathstr]

    proc_env = dict()

    for k in self.env.keys():
      v = self.env[k]
      print(k,v)
      value = "-D%s"%str(k)
      pe_val = "DEFINED"
      if isinstance(v,type(None)):
        pass
      if isinstance(v,bool):
        value += '=ON' if v else '=OFF'
        pe_val = 'ON' if v else 'OFF'
      else:
        value += "="+str(v)
        pe_val = str(v)
      proc_env[k] = pe_val
      cmdlist += [value]

    if self._trace:
      cmdlist += ["--trace"]

    if self._builddir==None:
      cmdlist += [str(self.root)]

    if self._xcode:
      cmdlist += ["-G","Xcode"]

    the_env = dict(os.environ)
    for k in self.osenv.keys():
      the_env[k] = str(self.osenv[k])
    #the_env["XXX"] = "xxx"

    with buildtrace.NestedBuildTrace({  
     "op": "cmake",
     "source_dir": self.root, 
     "build_dir": self._builddir,
     "prefix": self.install_prefix, 
     "module_path": self.install_prefix/"lib"/"cmake",
     "cmake_env": proc_env,
     "os_env": the_env }) as nested:
       return Command(cmdlist,environment=the_env,working_dir=self._working_dir).exec()



###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

VERSION = "v4.7.1"

import os, tarfile
from obt import dep, host, path, cmake, git, make, pathtools
from obt.deco import Deco
from obt.wget import wget
from obt.command import Command

deco = Deco()

###############################################################################
class cppzmq(dep.StdProvider):
  name = "cppzmq"
  def __init__(self):
    super().__init__(cppzmq.name)
    self.declareDep("cmake")
    self.declareDep("zmq")
    self._builder = self.createBuilder(dep.CMakeBuilder)
    self._builder.setCmVar("CPPZMQ_BUILD_TESTS","OFF")
###############################################################################
  def __str__(self):
    return "cppzmq (github-%s)" % VERSION
###############################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=cppzmq.name,
                             repospec="zeromq/cppzmq",
                             revision=VERSION,
                             recursive=True)
  ########################################################################
  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"CMakeLists.txt").exists()
  def areRequiredBinaryFilesPresent(self):
    return (path.includes()/"zmq.hpp").exists()



#!/usr/bin/env python3
from obt import command, docker

#command.Command(["docker","rm","-vf","$(docker ps -aq)"],do_log=True).exec(use_shell=True);
#command.Command(["docker","rmi","-f","$(docker images -aq)"],do_log=True).exec(use_shell=True);

container_list = docker.enumerate_all_continaers()
image_list = docker.enumerate_all_images()
print("containers: %s"%container_list)
print("images: %s" % image_list)

for item in container_list:
  command.run(["docker","kill",item],do_log=True)
for item in container_list:
  command.run(["docker","rm","-vf", item],do_log=True)  

for item in image_list:
  command.run(["docker","rmi","-f", item],do_log=True)  

command.run(["docker", "system", "prune", "-a"],do_log=True)


import obt.deco
from obt import path, dep
from obt.command import Command

def _command(name_or_list,cmd):
  #Command().exec()
  #pip_executable = path.stage()/"bin"/"pip3"
  PYTHON = dep.instance("python")
  python_executable = PYTHON.executable
  cmd_prefix = [python_executable,"-m","pip",cmd]

  rval = 0
  if(isinstance(name_or_list,str)):
    r = Command(cmd_prefix+[name_or_list]).exec()
    rval = r
  elif (isinstance(name_or_list,list)):
    r = Command(cmd_prefix+name_or_list).exec()
    rval = r

  return rval

def install(name_or_list):
  return _command(name_or_list,"install")

def uninstall(name_or_list):
  return _command(name_or_list,"uninstall")



from obt import dep, path, command, docker, wget, pathtools
from obt.deco import Deco
import obt.module
import time, re, socket, os, sys
from pathlib import Path
deco = obt.deco.Deco()

this_path = os.path.realpath(__file__)
this_dir = Path(os.path.dirname(this_path))

makedotcfg = this_dir/"Makefile.cfg"

###############################################################################

class dockerinfo:
    ###############################################
    def __init__(self):
      super().__init__()
      self.type = docker.Type.SINGLE
      self._name = "tflow2"
      self.imagename = "obt-tflow2:latest"
    ###############################################
    # build the docker image
    ###############################################
    def build(self,build_args):
      os.chdir(this_dir)
      UID = os.getuid()
      GID = os.getgid()
      cmdlist = ["docker", "build"]
      cmdlist += ["--build-arg", "UID=%d"%UID]
      cmdlist += ["--build-arg", "GID=%d"%GID]
      cmdlist += [".", "-t", self.imagename]
      command.run(cmdlist)
    ###############################################
    # kill active docker container
    ###############################################
    def kill(self):
      pass
    ###############################################
    # launch docker container
    #  print out connection info
    ###############################################
    def launch(self,launch_args):
    
        command.run(["xhost","+localhost"])

        command.run([
            "docker", "run",
            "-it",
            "--gpus","all",
            "--mount","type=bind,source=%s/test_single_gpu.py,target=/home/tensorflow/test_single_gpu.py"%this_dir,
            "--mount","type=bind,source=%s/test_multi_gpu.py,target=/home/tensorflow/test_multi_gpu.py"%this_dir,
            "--mount","type=bind,source=%s/test_mediapipe.sh,target=/home/tensorflow/test_mediapipe.sh"%this_dir,
            "-v","/tmp/.X11-unix:/tmp/.X11-unix",
            "-e","DISPLAY=%s"%os.environ["DISPLAY"],
            "-e","QT_X11_NO_MITSHM=1",
            #"--mount","type=bind,source=%s,target=/home/tensorflow/Makefile.cfg"%makedotcfg,
            self.imagename,
            "/bin/bash"
        ])
    ###############################################
    # information dictionary
    ###############################################
    def info(self):
      return {
        "image_name": self.imagename,
      }
    ###############################################


import obt.command

def shell(cmdlist=None,cpu=None,platform=None,target=None,triple=None):
  if triple!=None:
      cpu = triple["cpu"]
      platform = triple["platform"]
      target = triple["target"]
      obt.command.run(["obt.litex.env.py",
                       "--cpu",cpu,
                       "--platform",platform,
                       "--target",target,
                       "--shell"])

def run(cmdlist=None,cpu=None,platform=None,target=None,triple=None):
  if triple!=None:
      cpu = triple["cpu"]
      platform = triple["platform"]
      target = triple["target"]

  if isinstance(cmdlist,list):
    tmplist = []
    for item in cmdlist:
      tmplist.append(str(item))
    cmdlist = " ".join(tmplist)

    obt.command.run(["obt.litex.env.py",
                    "--cpu",cpu,
                    "--platform",platform,
                    "--target",target,
                    "--exec", cmdlist ])



import json, pprint, ast, json, sys, jsonpickle, threading, time
from pathlib import Path
from yarl import URL
from enum import Enum
pp = pprint.PrettyPrinter(indent=4,depth=4)
jsonpickle.set_preferred_backend('json')
jsonpickle.set_encoder_options('json', sort_keys=True, indent=3)

################################################################################
def DEBUGFORMAT(obj):
  a = jsonpickle.encode(obj,max_depth=2)
  return str(a)

################################################################################
class BranchStatus(Enum):
  INIT = 1
  BUILDING = 2
  PASSED = 3
  FAILED = 4
  WAIT = 5
################################################################################
class WorkerStatus(Enum):
  INIT = 1
  FETCHING = 2
  BUILDING = 3
  WAIT = 4
################################################################################
class Branch:
  def __init__(self):
    self.name = None
    self.repo_obj = None
    self.remote_giturl = None
    self.commits = None
    self.last_sha = None
    self.last_author = None
    self.last_data = None
    self.last_commit_url = None
################################################################################
"""
def update_badge(item):

 ##################################
 # set text
 ##################################

 if item["status"]==BranchStatus.PASSED:
   right_text = "PASSED"
 elif item["status"]==BranchStatus.FAILED:
   right_text = "FAILED"
 elif item["status"]==BranchStatus.INIT:
   right_text = "INIT"
 elif item["status"]==BranchStatus.BUILDING:
   right_text = "BUILDING"
 else:
   right_text = "????"

 ##################################
 # set color
 ##################################

 right_color = "#000000"

 if item["status"]==BranchStatus.PASSED:
   right_color = "#006000"
 elif item["status"]==BranchStatus.FAILED:
   right_color = "#500000"
 elif item["status"]==BranchStatus.INIT:
   right_color = "#000000"
 elif item["status"]==BranchStatus.BUILDING:
   if item["prev_status"]==BranchStatus.PASSED:
     right_color = "#006000"
   elif item["prev_status"]==BranchStatus.FAILED:
     right_color = "#500000"
   elif item["prev_status"]==BranchStatus.INIT:
     right_color = "#404040"

 print("NEWBRANCHSTATUS<%s>"%item["status"])
 ##################################
 # generate badge
 ##################################

 SHA = item["lastsha"]
 shadir = BUILDSBASEDIR/SHA
 os.system("mkdir -p %s"%str(shadir))
 badge.create( outpath=str(shadir/"status.svg"),
               left_text="BranchStatus",right_text=right_text,
               left_color="#202040",right_color=right_color)
"""



###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################
from obt import dep, log, path, host
###############################################################################
class _embree_from_source(dep.StdProvider):
  name = "embree"
  def __init__(self,name):
    super().__init__(_embree_from_source.name)
    self.declareDep("cmake")
    self._archlist = ["x86_64"]
    self.VERSION = "v3.9.0"
    self._builder = self.createBuilder(dep.CMakeBuilder)
    self._builder.setCmVar("EMBREE_TUTORIALS","FALSE") # because they dont compile with ispc==1.13.0

  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=_embree_from_source.name,
                             repospec="embree/embree",
                             revision=self.VERSION,
                             recursive=False)
  ########################################################################

  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"CMakeLists.txt").exists()

  def areRequiredBinaryFilesPresent(self):
    return (path.libs()/"libembree3.so").exists()
###############################################################################
class _embree_from_homebrew(dep.HomebrewProvider):
  def __init__(self,name):
    super().__init__(name,name)
    self.VERSION = "homebrew"
###############################################################################
class embree(dep.switch(linux=_embree_from_source, \
                        macos=_embree_from_homebrew)):
  def __init__(self):
    super().__init__("embree")
    if host.IsX86_64:
      self.requires(["ispc"])
  def env_init(self):
    log.marker("registering embree SDK(%s)"%self.VERSION)



###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

VERSION = ["1","81","0"]
HASH = "ffac94fbdd92d6bc70a897052022eeba"

import os,tarfile
from obt import path,host,dep, gen_pkgconfig, patch
from obt.wget import wget
from obt.deco import Deco
from obt.command import Command
from pathlib import Path
from yarl import URL

deco = Deco()

###########################################

class boost(dep.Provider):

  ########

  #https://github.com/boostorg/boost/releases/download/boost-1.81.0/boost-1.81.0.tar.gz
  #boost-1.81.0/boost-1.81.0.tar.gz

  def __init__(self):
    super().__init__("boost")
    self.version = VERSION
    self.baseurl = URL("https://github.com/boostorg/boost/releases/download")
    self.verurl = self.baseurl/("boost-%s.%s.%s"%(VERSION[0],VERSION[1],VERSION[2]))
    self.fbase = ("boost-%s.%s.%s"%(VERSION[0],VERSION[1],VERSION[2]))
    self.fname = ("%s.tar.gz"%(self.fbase))
    self.verurl = self.verurl/self.fname
    build_dest = path.builds()/"boost"
    self.build_dest = build_dest
    self._is_mac_arm = (self._target.identifier == "aarch64-macos")

    SUFFIX = ""
    if self._target.architecture == "x86_64":
      SUFFIX = "x64"
    elif self._target.architecture == "aarch64":
      SUFFIX = "a64"

    self.architecture_suffix = SUFFIX

  ########

  def __str__(self):
    return "Boost ver:%s" % VERSION

  ########

  def wipe(self):
    os.system("rm -rf %s"%self.source_root)

  ########

  def download_and_extract(self):

    self.arcpath = dep.downloadAndExtract([self.verurl],
                                          self.fname,
                                          "gz",
                                          HASH,
                                          self.build_dest)

  ########

  def build(self):

    PYTHON = dep.instance("python")

    prefix = path.prefix()
    toolset = "darwin" if self._target.os=="macos" else "gcc"

    os.chdir(str(self.build_dest/self.fbase))

    #########################################
    # for MacM1 (ARM)
    #  patch darwin.jam
    #  see https://github.com/boostorg/build/commit/456be0b7ecca065fbccf380c2f51e0985e608ba0.patch?full_index=1
    #  and https://trac.macports.org/ticket/60287
    #########################################

    if toolset == "darwin":
      p = patch.patcher(self)
      p_dest = self.build_dest/self.fbase/"tools"/"build"/"src"/"tools"
      p.patch(p_dest,"darwin.jam")

    #########################################

    cmdlist = [
     "./bootstrap.sh",
     "link=shared",
     "runtime-link=shared",
     "--prefix=%s"%prefix,
     "toolset=%s" % toolset
    ]

    #########################################
    # giving up on boost-python on mac M1 for now...
    #########################################

    if True: #toolset == "darwin":
      cmdlist += ["--without-libraries=python,coroutine"]
    else:
      # broken with python 3.12...
      cmdlist += [
       "--with-python=%s"%PYTHON.executable,
       "--with-python-root=%s"%PYTHON.home_dir,
       "--with-python-version=%s"%(PYTHON.version_major),
    ]

    #########################################

    a = Command(cmdlist).exec()

    OK = (a==0)
    assert(OK)

    #########################################
    # for MacM1 (ARM)
    #  patch project-config.jam
    #  as it cannot correctly find python
    #  see:  https://github.com/boostorg/build/issues/289
    #########################################

    #if self._is_mac_arm:
    #  repl = {
    #    "PY_VER": PYTHON.version_major,
    #    "PY_ROOT": PYTHON.home_dir,
    #    "PY_INC": PYTHON.include_dir,
    #    "PREFIX": path.stage(),
    #  }
    #  p = patch.patcher(self,repl_dict = repl)
    #  p.patch(self.build_dest/self.fbase,"project-config.jam")

    #########################################

    cxxflags = ["-std=c++17","-fPIC"]

    #########################################

    b = Command(["./b2",
                 "--prefix=%s"%prefix,
                 "toolset=%s" % toolset,
                 "link=shared",
                 "runtime-link=shared",
                 "headers"]).exec()

    OK = (b==0)
    assert(OK)

    #########################################

    if host.IsOsx:
      linkflags = ['-Wl,-rpath',str(path.prefix()/"lib")]
      linkflags += ["-stdlib=libc++"]
    else:
      linkflags = ['-Wl,-rpath',str(path.prefix()/"lib")]

    c = Command(["./b2",
                 "--prefix=%s"%prefix,
                 "-d2",
                 "-j%d"%host.NumCores,
                 "-sNO_LZMA=1",
                 "--layout=tagged",
                 "toolset=%s" % toolset,
                 "threading=multi",
                 "address-model=64",
                 'cxxflags=%s' % " ".join(cxxflags),
                 'linkflags=%s' % " ".join(linkflags),
                 "link=shared",
                 "runtime-link=shared",
                 "install"]).exec()

    OK = (c==0)
    assert(OK)

    #########################################

    if OK:
      self.manifest.touch()

    return OK
    #Fixed it by commenting the following in boost/tools/build/src/tools/darwin.jam
    # - GCC 4.0 and higher in Darwin does not have -fcoalesce-templates.
    #if $(real-version) < "4.0.0"
    #{
    #  flags darwin.compile.c++ OPTIONS $(condition) : -fcoalesce-templates ;
    #}

  ###########################################################

  def version(self):
      return "%s.%s.%s"%(VERSION[0],VERSION[1],VERSION[2])

  ###########################################################
  # generate pkgconfig package file
  ###########################################################

  def _generate_pkgconfig(self):
    LIBS_PUBLIC = """
    prg_exec_monitor 
    math_c99f unit_test_framework.
    container log_setup math_tr1l 
    graph wserialization log 
    math_c99f type_erasure signals 
    test_exec_monitor filesystem thread 
    math_tr1f date_time timer 
    math_tr1f test_exec_monitor container 
    math_tr1 type_erasure 
    program_options graph log_setup 
    random system system locale 
    wserialization regex exception 
    timer signals filesystem 
    math_c99 math_tr1 serialization 
    serialization prg_exec_monitor exception 
    coroutine math_c99 iostreams 
    random program_options atomic 
    date_time math_c99l math_tr1l 
    context regex coroutine 
    log chrono wave 
    iostreams chrono unit_test_framework.
    math_c99l
    """.replace("\n","")

    LIBS_PUBLIC = LIBS_PUBLIC.split(" ")
    while("" in LIBS_PUBLIC) :
      LIBS_PUBLIC.remove("")

    SUFFIX = "mt-%s" % self.architecture_suffix

    LIBS_PUBLIC = ["-lboost_%s-%s"%(item,SUFFIX) for item in LIBS_PUBLIC]

    CFLAGS = "-I%s/boost" % path.includes()

    replacements = {
      "NAME": "boost",
      "DESCRIPTION": "OBT boost",
      "VERSION": ("%s.%s.%s"%(VERSION[0],VERSION[1],VERSION[2])),
      "PREFIX": path.stage(),
      "LIBS_PRIVATE":"",
      "LIBS_PUBLIC":"-L%s %s"%(path.libs()," ".join(LIBS_PUBLIC)),
      "CFLAGS":CFLAGS,
    }

    if self._target.os != "macos":
      g = gen_pkgconfig.Generator()
      a = g.apply(replacements,outpath=path.pkgconfigdir()/"boost.pc")

    #############################
    # change install names
    #############################

    if self._target.os == "macos":
      print( "MACOS - fixing boost installnames")
      from obt import macos
      libs = [
        "filesystem",
        "system",
        "program_options",
        "thread",
        "chrono",
        "date_time",
        "atomic",
      ]
      for l in libs:
        if self._target.architecture == "aarch64":
          l2 = "libboost_%s-mt-%s.dylib" % (l,self.architecture_suffix)
        else:
          l2 = "libboost_%s-mt-x64.dylib" % (l)
        print(l,l2)
        macos.macho_change_id(path.libs()/l2,"@rpath/%s"%l2)

  ###########################################################

  def cmake_additional_flags(self):
    return {
      "Boost_NO_SYSTEM_PATHS": "ON",
      "BOOST_ROOT": path.stage(),
      #"Boost_DEBUG": "ON",
      "Boost_USE_STATIC_LIBS": "OFF",
      "BOOST_LIBRARYDIR": path.libs(),
      "BOOST_INCLUDEDIR": path.includes(),
      "Boost_ARCHITECTURE": "-%s" % self.architecture_suffix,
    }

  ###########################################################

  def provide(self):
    OK = self.manifest.exists()
    if not self.supports_host:
      print(deco.red("Dependency does not support this host"))
      return False
    ############################
    if self.should_wipe:
      self.wipe()
    ############################
    src_present = self.areRequiredSourceFilesPresent()
    if not src_present:
      self.download_and_extract()
    ############################
    if self.should_build:
      OK = self.build()
    ############################
    self._generate_pkgconfig()

    return OK

  ###########################################################

  def areRequiredSourceFilesPresent(self):
    return (self.build_dest/self.fbase/"boostcpp.jam").exists()

  ###########################################################

  def areRequiredBinaryFilesPresent(self):
    return (path.includes()/"boost"/"blank.hpp").exists()



###############################################################################
# Orkid Build System
# Copyright 2010-2022, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import platform, os, pathlib,sys
from obt import host
###############################################################################

file_path = os.path.realpath(__file__)
this_dir = pathlib.Path(os.path.dirname(file_path))

###############################################################################

def descriptor(architecture,osname):
  import obt.module
  import obt.path
  identifier = "%s-%s" % (architecture,osname)
  hi_name = obt.path.obt_modules_base()/"target"
  hi_name = hi_name / ("%s.py"%identifier)
  the_module = obt.module.instance(identifier,hi_name)
  if the_module != None:
    return the_module.targetinfo()

###############################################################################

def current():
  tgt_name = os.environ.get("OBT_TARGET")
  d = None
  if tgt_name==None:
    d = host.description().identifier.split("-")
  else:
    d = tgt_name.split("-")
  if len(d)==2:
    return descriptor(d[0],d[1])
###############################################################################

class enuminterface:
  def __init__(self):
    self.subdir = "target"
  def tryAsModule(self,item,pth):
    return None

###############################################################################

def enumerate():
  iface = enuminterface()
  return obt.module.enumerate_simple(iface)



###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

VERSION = "v4.7.1"

import os, tarfile
from obt import dep, host, path, cmake, git, make, pathtools
from obt.deco import Deco
from obt.wget import wget
from obt.command import Command

deco = Deco()

###############################################################################
class depot_tools(dep.StdProvider):
  name = "depot_tools"
  def __init__(self):
    super().__init__(depot_tools.name)
    self.declareDep("cmake")
    self.declareDep("zmq")
    self._builder = self.createBuilder(dep.NopBuilder)
###############################################################################
  def __str__(self):
    return "depot_tools (github-%s)" % VERSION
###############################################################################
  @property
  def _binpath(self):
    return path.builds()/"depot_tools"
###############################################################################
  @property
  def _fetcher(self):
    fetcher = dep.GitFetcher(name=depot_tools.name)
    fetcher._git_url = "https://chromium.googlesource.com/chromium/tools/depot_tools.git"
    fetcher._revision = "main"
    return fetcher
  ########################################################################
  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"CMakeLists.txt").exists()
  def areRequiredBinaryFilesPresent(self):
    return (path.includes()/"zmq.hpp").exists()



#!/usr/bin/env python3
###############################################################################
# Orkid Build System
# Copyright 2010-2022, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os, sys, pathlib, argparse, multiprocessing, json

as_main = (__name__ == '__main__')

Path = pathlib.Path
curwd = Path(os.getcwd())
###########################################

file_path = os.path.realpath(__file__)
file_dir = os.path.dirname(file_path)
par2_dir = os.path.dirname(file_dir)

root_dir = Path(par2_dir)
scripts_dir = root_dir/"scripts"
project_dir = root_dir

sys.path.append(str(file_dir))

###########################################

parser = argparse.ArgumentParser(description='obt.build environment explicit tester')
parser.add_argument('--stagedir', metavar="stagedir", help='staging folder for session' )
parser.add_argument('--projectdir',metavar="projectdir",help='sidechain project directory(with obt manifest)')
parser.add_argument('--inplace', action="store_true" )
parser.add_argument("--quiet", action="store_true", help="no output")
parser.add_argument("--command", metavar="command", help="execute in environ")
parser.add_argument("--numcores", metavar="numcores", help="numcores for environment")

###########################################

args = vars(parser.parse_args())

if len(sys.argv)==1:
    print(parser.format_usage())
    sys.exit(1)

###########################################

from _obt_config import configFromCommandLine
config = configFromCommandLine(args)
config.dump()

